import os
import shutil
import sys
import tempfile
import unittest

import configbetter


class TestMakedirs(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        configbetter.sys.platform = sys.platform
        cls.tempdir = tempfile.mkdtemp()
        if sys.platform in ['linux', 'darwin']:
            configbetter.os.environ['HOME'] = cls.tempdir
        elif sys.platform == 'win32':
            configbetter.os.environ['APPDATA'] = cls.tempdir
        if 'XDG_DATA_HOME' in configbetter.os.environ:
            del configbetter.os.environ['XDG_DATA_HOME']
        if 'XDG_CONFIG_HOME' in configbetter.os.environ:
            del configbetter.os.environ['XDG_CONFIG_HOME']
        if 'XDG_CACHE_HOME' in configbetter.os.environ:
            del configbetter.os.environ['XDG_CACHE_HOME']
        cls.conf = configbetter.Config('notarealapp')
        cls.conf.makedirs()

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(cls.tempdir)

    def test_data_dir(self):
        self.assertTrue(os.path.exists(self.conf.data))

    def test_config_dir(self):
        self.assertTrue(os.path.exists(self.conf.config))

    def test_cache_dir(self):
        self.assertTrue(os.path.exists(self.conf.cache))

class TestRmdirs(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        configbetter.sys.platform = 'win32'
        cls.tempdir = tempfile.mkdtemp()
        if sys.platform in ['linux', 'darwin']:
            configbetter.os.environ['HOME'] = cls.tempdir
        elif sys.platform == 'win32':
            configbetter.os.environ['APPDATA'] = cls.tempdir
        if 'XDG_DATA_HOME' in configbetter.os.environ:
            del configbetter.os.environ['XDG_DATA_HOME']
        if 'XDG_CONFIG_HOME' in configbetter.os.environ:
            del configbetter.os.environ['XDG_CONFIG_HOME']
        if 'XDG_CACHE_HOME' in configbetter.os.environ:
            del configbetter.os.environ['XDG_CACHE_HOME']
        cls.conf = configbetter.Config('notarealapp')
        cls.conf.makedirs()
        cls.conf.rmdirs()

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(cls.tempdir)

    def test_data_rmdir(self):
        self.assertFalse(os.path.exists(self.conf.data))

    def test_config_rmdir(self):
        self.assertFalse(os.path.exists(self.conf.config))

    def test_cache_rmdir(self):
        self.assertFalse(os.path.exists(self.conf.cache))

    def test_windows_edgecase_rmdir(self):
        self.assertFalse(os.path.exists(os.path.join(self.tempdir, 'notarealapp')))


class TestWindowsNoXDG(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        configbetter.sys.platform = 'win32'
        cls.tempdir = tempfile.mkdtemp()
        configbetter.os.environ['APPDATA'] = cls.tempdir
        cls.conf = configbetter.Config('fakeapp')

    @classmethod
    def tearDownClass(cls):
        os.rmdir(cls.tempdir)

    def test_noxdg_data(self):
        checkdata = os.path.join(self.tempdir, 'fakeapp', 'Data')
        self.assertEqual(self.conf.data, checkdata)

    def test_noxdg_config(self):
        checkconfig = os.path.join(self.tempdir, 'fakeapp', 'Config')
        self.assertEqual(self.conf.config, checkconfig)

    def test_noxdg_cache(self):
        checkcache = os.path.join(self.tempdir, 'fakeapp', 'Cache')
        self.assertEqual(self.conf.cache, checkcache)


class TestLinuxNoXDG(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        configbetter.sys.platform = 'linux'
        cls.tempdir = tempfile.mkdtemp()
        configbetter.os.environ['HOME'] = cls.tempdir
        cls.conf = configbetter.Config('fakeapp')

    @classmethod
    def tearDownClass(cls):
        os.rmdir(cls.tempdir)

    def test_noxdg_data(self):
        checkdata = os.path.join(self.tempdir, '.local', 'share', 'fakeapp')
        self.assertEqual(self.conf.data, checkdata)

    def test_noxdg_config(self):
        checkconfig = os.path.join(self.tempdir, '.config', 'fakeapp')
        self.assertEqual(self.conf.config, checkconfig)

    def test_noxdg_cache(self):
        checkcache = os.path.join(self.tempdir, '.cache', 'fakeapp')
        self.assertEqual(self.conf.cache, checkcache)


class TestMacNoXDG(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        configbetter.sys.platform = 'darwin'
        cls.tempdir = tempfile.mkdtemp()
        configbetter.os.environ['HOME'] = cls.tempdir
        cls.conf = configbetter.Config('fakeapp')

    @classmethod
    def tearDownClass(cls):
        os.rmdir(cls.tempdir)

    def test_noxdg_data(self):
        checkdata = os.path.join(self.tempdir, 'Library', 'fakeapp')
        self.assertEqual(self.conf.data, checkdata)

    def test_noxdg_config(self):
        checkconfig = os.path.join(self.tempdir, 'Library', 'Preferences', 'fakeapp')
        self.assertEqual(self.conf.config, checkconfig)

    def test_noxdg_cache(self):
        checkcache = os.path.join(self.tempdir, 'Library', 'Caches', 'fakeapp')
        self.assertEqual(self.conf.cache, checkcache)


class TestXDG(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        configbetter.sys.platform = 'win32'
        cls.datatempdir = tempfile.mkdtemp()
        cls.configtempdir = tempfile.mkdtemp()
        cls.cachetempdir = tempfile.mkdtemp()
        configbetter.os.environ['XDG_DATA_HOME'] = cls.datatempdir
        configbetter.os.environ['XDG_CONFIG_HOME'] = cls.configtempdir
        configbetter.os.environ['XDG_CACHE_HOME'] = cls.cachetempdir
        cls.conf = configbetter.Config('fakeapp')

    @classmethod
    def tearDownClass(cls):
        os.rmdir(cls.datatempdir)
        os.rmdir(cls.configtempdir)
        os.rmdir(cls.cachetempdir)

    def test_xdg_data(self):
        checkdata = os.path.join(self.datatempdir, 'fakeapp')
        self.assertEqual(self.conf.data, checkdata)

    def test_xdg_config(self):
        checkconfig = os.path.join(self.configtempdir, 'fakeapp')
        self.assertEqual(self.conf.config, checkconfig)

    def test_xdg_cache(self):
        checkcache = os.path.join(self.cachetempdir, 'fakeapp')
        self.assertEqual(self.conf.cache, checkcache)


if __name__ == '__main__':
    unittest.main()



"""
Handles all of the logic for config-better.
"""

import os
import shutil
import sys


def _getdir(*pathparts) -> str:
    return os.path.expandvars(os.path.join(*pathparts))


class Config:
    """
    Provides functionality to determine & create directories.
    """

    def __init__(self, appname: str):
        """
        Creates a Config object.
        
        This also pulls in environment data related to the XDG Base Directory
        specification.

        Arguments:
            appname {str} -- The name that will be used when deciding the
                             folders to be used for data, config and cache.
        """

        self.appname = appname
        self.__data = None
        self.__config = None
        self.__cache = None
        if 'XDG_DATA_HOME' in os.environ:
            self.__data = os.environ['XDG_DATA_HOME']
        if 'XDG_CONFIG_HOME' in os.environ:
            self.__config = os.environ['XDG_CONFIG_HOME']
        if 'XDG_CACHE_HOME' in os.environ:
            self.__cache = os.environ['XDG_CACHE_HOME']

    @property
    def data(self) -> str:
        """
        Provides the data folder the application should use. 
        
        To follow the XDG Base Directory specification, it first checks whether
        the XDG_DATA_HOME environment variable has been set.

        If set:
         - The data directory should be $XDG_DATA_HOME/appname

        Otherwise, it is platform dependent:
         - Windows: $APPDATA/appname/Data
         - Mac OS: $HOME/Library/appname
         - Linux/Other: $HOME/.local/share/appname

        Returns:
            str -- The full path to the preferred data folder.
        """

        if self.__data:
            return _getdir(self.__data, self.appname)
        if sys.platform == 'win32':
            return _getdir(os.environ['APPDATA'], self.appname, 'Data')
        if sys.platform == 'darwin':
            return _getdir('$HOME', 'Library', self.appname)
        return _getdir('$HOME', '.local', 'share', self.appname)

    @property
    def config(self) -> str:
        """
        Provides the config folder the application should use.
        
        To follow the XDG Base Directory specification, it first checks whether
        the XDG_CONFIG_HOMEenvironment variable has been set.

        If set:
         - The data directory should be $XDG_CONFIG_HOME/appname

        Otherwise, it is platform dependent:
         - Windows: $APPDATA/appname/Config
         - Mac OS: $HOME/Library/Preferences/appname
         - Linux/Other: $HOME/.config/appname

        Returns:
            str -- The full path to the preferred config folder.
        """

        if self.__config:
            return _getdir(self.__config, self.appname)
        if sys.platform == 'win32':
            return _getdir(os.environ['APPDATA'], self.appname, 'Config')
        if sys.platform == 'darwin':
            return _getdir('$HOME', 'Library', 'Preferences', self.appname)
        return _getdir('$HOME', '.config', self.appname)

    @property
    def cache(self) -> str:
        """
        Provides the cache folder the application should use. 
        
        To follow the XDG Base Directory specification, it first checks whether
        the XDG_CACHE_HOME environment variable has been set.

        If set:
         - The data directory should be $XDG_CACHE_HOME/appname

        Otherwise, it is platform dependent:
         - Windows: $APPDATA/appname/Cache
         - Mac OS: $HOME/Library/Caches/appname
         - Linux/Other: $HOME/.cache/appname

        Returns:
            str -- The full path to the preferred cache folder.
        """

        if self.__cache:
            return _getdir(self.__cache, self.appname)
        if sys.platform == 'win32':
            return _getdir(os.environ['APPDATA'], self.appname, 'Cache')
        if sys.platform == 'darwin':
            return _getdir('$HOME', 'Library', 'Caches', self.appname)
        return _getdir('$HOME', '.cache', self.appname)

    def makedirs(self):
        """
        Creates the data, cache, and config folders.

        New directories are only made if they do not already exist.
        """

        if not os.path.exists(self.data):
            os.makedirs(self.data)
        if not os.path.exists(self.config):
            os.makedirs(self.config)
        if not os.path.exists(self.cache):
            os.makedirs(self.cache)

    def rmdirs(self):
        """
        Removes the data, cache and config folders.

        May be useful for uninstall scripts or cleaning scripts. Handles a
        special case for Windows where, in default scenarios, there is a parent]
        directory that needs to be cleared.
        """

        if os.path.exists(self.cache):
            shutil.rmtree(self.cache)
        if os.path.exists(self.config):
            shutil.rmtree(self.config)
        if os.path.exists(self.data):
            shutil.rmtree(self.data)
        windows_dir = _getdir(os.environ['APPDATA'], self.appname)
        if sys.platform == 'win32' and not (self.__data or self.__cache
                                            or self.__config) and os.path.exists(windows_dir):
            shutil.rmtree(windows_dir)



import subprocess

from setuptools import find_packages, setup

import configbetter

with open('README.md') as fh:
    long_desc = fh.read()

version = "1.0.0"
last_commit = subprocess.check_output(["git", "rev-list", "HEAD",
                                       "--count"]).decode('utf-8').strip()
branch = subprocess.check_output(["git", "rev-parse", "--abbrev-ref",
                                  "HEAD"]).decode('utf-8').strip()

setup(
    name="config-better",
    version=version + (f".{last_commit}-{branch}" if __debug__ else ""),
    description="Configure your application in a friendlier and more consistent way!",
    long_description=long_desc,
    long_description_content_type='text/markdown',
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    license="MIT",
    url="https://github.com/kade-robertson/config-better",
    keywords="config better xdg directory",
    author="Kade Robertson",
    author_email="kade@kaderobertson.pw",
    packages=find_packages(),
    install_requires=[],
    test_suite='tests',
    python_requires='>=3.5.*')
